@@ -34,10 +34,10 @@
 #define _WIN32_WINNT 0x0501
 
 #define PACKAGE "BitlBee"
-#define BITLBEE_VERSION "1.3dev"
+#define BITLBEE_VERSION "3.0.1"
 #define VERSION BITLBEE_VERSION
 #define BITLBEE_VER(a,b,c) (((a) << 16) + ((b) << 8) + (c))
-#define BITLBEE_VERSION_CODE BITLBEE_VER(1, 3, 0)
+#define BITLBEE_VERSION_CODE BITLBEE_VER(3, 0, 1)
 
 #define MAX_STRING 511
 
@@ -125,6 +125,10 @@
 #define HELP_FILE VARDIR "help.txt"
 #define CONF_FILE_DEF ETCDIR "bitlbee.conf"
 
+/* Hack to give a little bit more password security on IRC: If an account has
+   this password set, use /OPER to change it. */
+#define PASSWORD_PENDING "\r\rchangeme\r\r"
+
 #include "bee.h"
 #include "irc.h"
 #include "storage.h"
@@ -136,7 +140,6 @@
 #include "conf.h"
 #include "log.h"
 #include "ini.h"
-#include "help.h"
 #include "query.h"
 #include "sock.h"
 #include "misc.h"
@@ -146,7 +149,7 @@ typedef struct global {
 	/* In forked mode, child processes store the fd of the IPC socket here. */
 	int listen_socket;
 	gint listen_watch_source_id;
-	help_t *help;
+	struct help *help;
 	char *conf_file;
 	conf_t *conf;
 	GList *storage; /* The first backend in the list will be used for saving */
@@ -162,6 +165,7 @@ gboolean bitlbee_io_current_client_write( gpointer data, gint source, b_input_co
 
 void root_command_string( irc_t *irc, char *command );
 void root_command( irc_t *irc, char *command[] );
+gboolean root_command_add( const char *command, int params, void (*func)(irc_t *, char **args), int flags );
 gboolean cmd_identify_finish( gpointer data, gint fd, b_input_condition cond );
 gboolean bitlbee_shutdown( gpointer data, gint fd, b_input_condition cond );
 
