@@ -28,6 +28,7 @@
 #include "dcc.h"
 
 GSList *irc_connection_list;
+GSList *irc_plugins;
 
 static gboolean irc_userping( gpointer _irc, gint fd, b_input_condition cond );
 static char *set_eval_charset( set_t *set, char *value );
@@ -41,6 +42,7 @@ irc_t *irc_new( int fd )
 	socklen_t socklen = sizeof( sock );
 	char *host = NULL, *myhost = NULL;
 	irc_user_t *iu;
+	GSList *l;
 	set_t *s;
 	bee_t *b;
 	
@@ -102,12 +104,15 @@ irc_t *irc_new( int fd )
 	
 	s = set_add( &b->set, "allow_takeover", "true", set_eval_bool, irc );
 	s = set_add( &b->set, "away_devoice", "true", set_eval_bw_compat, irc );
+	s->flags |= SET_HIDDEN;
 	s = set_add( &b->set, "away_reply_timeout", "3600", set_eval_int, irc );
 	s = set_add( &b->set, "charset", "utf-8", set_eval_charset, irc );
 	s = set_add( &b->set, "default_target", "root", NULL, irc );
 	s = set_add( &b->set, "display_namechanges", "false", set_eval_bool, irc );
 	s = set_add( &b->set, "display_timestamps", "true", set_eval_bool, irc );
 	s = set_add( &b->set, "handle_unknown", "add_channel", NULL, irc );
+	s = set_add( &b->set, "last_version", "0", NULL, irc );
+	s->flags |= SET_HIDDEN;
 	s = set_add( &b->set, "lcnicks", "true", set_eval_bool, irc );
 	s = set_add( &b->set, "nick_format", "%-@nick", NULL, irc );
 	s = set_add( &b->set, "offline_user_quits", "true", set_eval_bool, irc );
@@ -117,11 +122,13 @@ irc_t *irc_new( int fd )
 	s = set_add( &b->set, "paste_buffer_delay", "200", set_eval_int, irc );
 	s->old_key = g_strdup( "buddy_sendbuffer_delay" );
 	s = set_add( &b->set, "password", NULL, set_eval_password, irc );
-	s->flags |= SET_NULL_OK;
+	s->flags |= SET_NULL_OK | SET_PASSWORD;
 	s = set_add( &b->set, "private", "true", set_eval_bool, irc );
 	s = set_add( &b->set, "query_order", "lifo", NULL, irc );
 	s = set_add( &b->set, "root_nick", ROOT_NICK, set_eval_root_nick, irc );
+	s->flags |= SET_HIDDEN;
 	s = set_add( &b->set, "show_offline", "false", set_eval_bw_compat, irc );
+	s->flags |= SET_HIDDEN;
 	s = set_add( &b->set, "simulate_netsplit", "true", set_eval_bool, irc );
 	s = set_add( &b->set, "timezone", "local", set_eval_timezone, irc );
 	s = set_add( &b->set, "to_char", ": ", set_eval_to_char, irc );
@@ -156,7 +163,18 @@ irc_t *irc_new( int fd )
 	g_free( myhost );
 	g_free( host );
 	
+	/* libpurple doesn't like fork()s after initializing itself, so this
+	   is the right moment to initialize it. */
+#ifdef WITH_PURPLE
 	nogaim_init();
+#endif
+	
+	for( l = irc_plugins; l; l = l->next )
+	{
+		irc_plugin_t *p = l->data;
+		if( p->irc_new )
+			p->irc_new( irc );
+	}
 	
 	return irc;
 }
@@ -177,7 +195,8 @@ void irc_abort( irc_t *irc, int immed, char *format, ... )
 		va_end( params );
 	}
 	
-	irc_write( irc, "ERROR :Closing link: %s", reason ? : "" );
+	if( reason )
+		irc_write( irc, "ERROR :Closing link: %s", reason );
 	
 	ipc_to_master_str( "OPERMSG :Client exiting: %s@%s [%s]\r\n",
 	                   irc->user->nick ? irc->user->nick : "(NONE)",
@@ -201,6 +220,8 @@ static gboolean irc_free_hashkey( gpointer key, gpointer value, gpointer data );
 
 void irc_free( irc_t * irc )
 {
+	GSList *l;
+	
 	irc->status |= USTATUS_SHUTDOWN;
 	
 	log_message( LOGLVL_INFO, "Destroying connection with fd %d", irc->fd );
@@ -209,6 +230,13 @@ void irc_free( irc_t * irc )
 		if( storage_save( irc, NULL, TRUE ) != STORAGE_OK )
 			log_message( LOGLVL_WARNING, "Error while saving settings for user %s", irc->user->nick );
 	
+	for( l = irc_plugins; l; l = l->next )
+	{
+		irc_plugin_t *p = l->data;
+		if( p->irc_free )
+			p->irc_free( irc );
+	}
+	
 	irc_connection_list = g_slist_remove( irc_connection_list, irc );
 	
 	while( irc->queries != NULL )
@@ -681,6 +709,9 @@ void irc_sync( irc_t *irc )
 		if( ic->flags & IRC_CHANNEL_JOINED )
 			irc_send_join( ic, irc->user );
 	}
+	
+	/* We may be waiting for a PONG from the previous client connection. */
+	irc->pinging = FALSE;
 }
 
 void irc_desync( irc_t *irc )
@@ -914,7 +945,7 @@ static char *set_eval_bw_compat( set_t *set, char *value )
 	else if( strcmp( set->key, "show_offline" ) == 0 && bool2int( value ) )
 		val = "online@,away+,offline";
 	else
-		return SET_INVALID;
+		val = "online+,away";
 	
 	for( l = irc->channels; l; l = l->next )
 	{
@@ -926,3 +957,8 @@ static char *set_eval_bw_compat( set_t *set, char *value )
 	
 	return SET_INVALID;
 }
+
+void register_irc_plugin( const struct irc_plugin *p )
+{
+	irc_plugins = g_slist_prepend( irc_plugins, (gpointer) p );
+}
