@@ -191,7 +191,7 @@ static char *set_eval_channel_type( set_t *set, char *value )
 	
 	if( strcmp( value, "control" ) == 0 )
 		new = &control_channel_funcs;
-	else if( strcmp( value, "chat" ) == 0 )
+	else if( ic != ic->irc->default_channel && strcmp( value, "chat" ) == 0 )
 		new = &irc_channel_im_chat_funcs;
 	else
 		return SET_INVALID;
@@ -291,6 +291,43 @@ irc_channel_user_t *irc_channel_has_user( irc_channel_t *ic, irc_user_t *iu )
 	return NULL;
 }
 
+/* Find a channel we're currently in, that currently has iu in it. */
+struct irc_channel *irc_channel_with_user( irc_t *irc, irc_user_t *iu )
+{
+	GSList *l;
+	
+	for( l = irc->channels; l; l = l->next )
+	{
+		irc_channel_t *ic = l->data;
+		
+		if( strcmp( set_getstr( &ic->set, "type" ), "control" ) != 0 )
+			continue;
+		
+		if( ( ic->flags & IRC_CHANNEL_JOINED ) &&
+		    irc_channel_has_user( ic, iu ) )
+			return ic;
+	}
+	
+	/* If there was no match, try once more but just see if the user
+	   *would* be in the channel, i.e. if s/he were online. */
+	if( iu->bu == NULL )
+		return NULL;
+	
+	for( l = irc->channels; l; l = l->next )
+	{
+		irc_channel_t *ic = l->data;
+		
+		if( strcmp( set_getstr( &ic->set, "type" ), "control" ) != 0 )
+			continue;
+		
+		if( ( ic->flags & IRC_CHANNEL_JOINED ) &&
+		    irc_channel_wants_user( ic, iu ) )
+			return ic;
+	}
+	
+	return NULL;
+}
+
 int irc_channel_set_topic( irc_channel_t *ic, const char *topic, const irc_user_t *iu )
 {
 	g_free( ic->topic );
@@ -725,6 +762,31 @@ fail:
 	return SET_INVALID;	
 }
 
+/* Figure out if a channel is supposed to have the user, assuming s/he is
+   online or otherwise also selected by the show_users setting. Only works
+   for control channels, but does *not* check if this channel is of that
+   type. Beware! */
+gboolean irc_channel_wants_user( irc_channel_t *ic, irc_user_t *iu )
+{
+	struct irc_control_channel *icc = ic->data;
+	
+	if( iu->bu == NULL )
+		return FALSE;
+	
+	switch( icc->type )
+	{
+	case IRC_CC_TYPE_GROUP:
+		return iu->bu->group == icc->group;
+	case IRC_CC_TYPE_ACCOUNT:
+		return iu->bu->ic->acc == icc->account;
+	case IRC_CC_TYPE_PROTOCOL:
+		return iu->bu->ic->acc->prpl == icc->protocol;
+	case IRC_CC_TYPE_DEFAULT:
+	default:
+		return TRUE;
+	}
+}
+
 static gboolean control_channel_free( irc_channel_t *ic )
 {
 	struct irc_control_channel *icc = ic->data;
@@ -733,6 +795,7 @@ static gboolean control_channel_free( irc_channel_t *ic )
 	set_del( &ic->set, "fill_by" );
 	set_del( &ic->set, "group" );
 	set_del( &ic->set, "protocol" );
+	set_del( &ic->set, "show_users" );
 	
 	g_free( icc );
 	ic->data = NULL;
