@@ -25,6 +25,7 @@
 
 #define BITLBEE_CORE
 #include "bitlbee.h"
+#include "help.h"
 #include "ipc.h"
 
 static void irc_cmd_pass( irc_t *irc, char **cmd )
@@ -37,7 +38,8 @@ static void irc_cmd_pass( irc_t *irc, char **cmd )
 		   command last. (Possibly it won't send it at all if it turns
 		   out we don't require it, which will break this feature.)
 		   Try to identify using the given password. */
-		return root_command( irc, send_cmd );
+		root_command( irc, send_cmd );
+		return;
 	}
 	/* Handling in pre-logged-in state, first see if this server is
 	   password-protected: */
@@ -300,11 +302,19 @@ static void irc_cmd_who( irc_t *irc, char **cmd )
 {
 	char *channel = cmd[1];
 	irc_channel_t *ic;
+	irc_user_t *iu;
 	
 	if( !channel || *channel == '0' || *channel == '*' || !*channel )
 		irc_send_who( irc, irc->users, "**" );
 	else if( ( ic = irc_channel_by_name( irc, channel ) ) )
 		irc_send_who( irc, ic->users, channel );
+	else if( ( iu = irc_user_by_name( irc, channel ) ) )
+	{
+		/* Tiny hack! */
+		GSList *l = g_slist_append( NULL, iu );
+		irc_send_who( irc, l, channel );
+		g_slist_free( l );
+	}
 	else
 		irc_send_num( irc, 403, "%s :No such channel", channel );
 }
@@ -390,6 +400,23 @@ static void irc_cmd_nickserv( irc_t *irc, char **cmd )
 
 static void irc_cmd_oper( irc_t *irc, char **cmd )
 {
+	account_t *a;
+	
+	/* /OPER can now also be used to enter IM passwords without echoing.
+	   It's a hack but the extra password security is worth it. */
+	for( a = irc->b->accounts; a; a = a->next )
+		if( strcmp( a->pass, PASSWORD_PENDING ) == 0 )
+		{
+			set_setstr( &a->set, "password", cmd[2] );
+			irc_usermsg( irc, "Password added to IM account "
+			             "%s(%s)", a->prpl->name, a->user );
+			/* The IRC client may expect this. Report failure since
+			   we didn't hand out a +o. */
+			irc_send_num( irc, 491, ":Password added to IM account "
+			              "%s(%s)", a->prpl->name, a->user );
+			return;
+		}
+	
 	if( global.conf->oper_pass &&
 	    ( strncmp( global.conf->oper_pass, "md5:", 4 ) == 0 ?
 	        md5_verify_password( cmd[2], global.conf->oper_pass + 4 ) == 0 :
@@ -400,7 +427,7 @@ static void irc_cmd_oper( irc_t *irc, char **cmd )
 	}
 	else
 	{
-		irc_send_num( irc, 432, ":Incorrect password" );
+		irc_send_num( irc, 491, ":Incorrect password" );
 	}
 }
 
@@ -624,8 +651,8 @@ static void irc_cmd_completions( irc_t *irc, char **cmd )
 	
 	irc_send_msg_raw( irc->root, "NOTICE", irc->user->nick, "COMPLETIONS OK" );
 	
-	for( i = 0; commands[i].command; i ++ )
-		irc_send_msg_f( irc->root, "NOTICE", irc->user->nick, "COMPLETIONS %s", commands[i].command );
+	for( i = 0; root_commands[i].command; i ++ )
+		irc_send_msg_f( irc->root, "NOTICE", irc->user->nick, "COMPLETIONS %s", root_commands[i].command );
 	
 	for( h = global.help; h; h = h->next )
 		irc_send_msg_f( irc->root, "NOTICE", irc->user->nick, "COMPLETIONS help %s", h->title );
