@@ -54,27 +54,28 @@ static gboolean bee_irc_user_new( bee_t *bee, bee_user_t *bu )
 	bu->ui_data = iu = irc_user_new( irc, nick );
 	iu->bu = bu;
 	
+	if( set_getbool( &irc->b->set, "private" ) )
+		iu->last_channel = NULL;
+	else
+		iu->last_channel = irc_channel_with_user( irc, iu );
+	
 	if( ( s = strchr( bu->handle, '@' ) ) )
 	{
 		iu->host = g_strdup( s + 1 );
 		iu->user = g_strndup( bu->handle, s - bu->handle );
 	}
-	else if( bu->ic->acc->server )
-	{
-		iu->host = g_strdup( bu->ic->acc->server );
-		iu->user = g_strdup( bu->handle );
-		
-		/* s/ /_/ ... important for AOL screennames */
-		for( s = iu->user; *s; s ++ )
-			if( *s == ' ' )
-				*s = '_';
-	}
 	else
 	{
-		iu->host = g_strdup( bu->ic->acc->prpl->name );
 		iu->user = g_strdup( bu->handle );
+		if( bu->ic->acc->server )
+			iu->host = g_strdup( bu->ic->acc->server );
+		else
+			iu->host = g_strdup( bu->ic->acc->prpl->name );
 	}
 	
+	while( ( s = strchr( iu->user, ' ' ) ) )
+		*s = '_';
+	
 	if( bu->flags & BEE_USER_LOCAL )
 	{
 		char *s = set_getstr( &bee->set, "handle_unknown" );
@@ -140,9 +141,7 @@ static gboolean bee_irc_user_status( bee_t *bee, bee_user_t *bu, bee_user_t *old
 
 void bee_irc_channel_update( irc_t *irc, irc_channel_t *ic, irc_user_t *iu )
 {
-	struct irc_control_channel *icc;
 	GSList *l;
-	gboolean match = FALSE;
 	
 	if( ic == NULL )
 	{
@@ -167,23 +166,13 @@ void bee_irc_channel_update( irc_t *irc, irc_channel_t *ic, irc_user_t *iu )
 		return;
 	}
 	
-	icc = ic->data;
-	
-	if( icc->type == IRC_CC_TYPE_DEFAULT )
-		match = TRUE;
-	else if( icc->type == IRC_CC_TYPE_GROUP )
-		match = iu->bu->group == icc->group;
-	else if( icc->type == IRC_CC_TYPE_ACCOUNT )
-		match = iu->bu->ic->acc == icc->account;
-	else if( icc->type == IRC_CC_TYPE_PROTOCOL )
-		match = iu->bu->ic->acc->prpl == icc->protocol;
-	
-	if( !match )
+	if( !irc_channel_wants_user( ic, iu ) )
 	{
 		irc_channel_del_user( ic, iu, IRC_CDU_PART, NULL );
 	}
 	else
 	{
+		struct irc_control_channel *icc = ic->data;
 		int mode = 0;
 		
 		if( !( iu->bu->flags & BEE_USER_ONLINE ) )
@@ -203,13 +192,15 @@ void bee_irc_channel_update( irc_t *irc, irc_channel_t *ic, irc_user_t *iu )
 	}
 }
 
-static gboolean bee_irc_user_msg( bee_t *bee, bee_user_t *bu, const char *msg, time_t sent_at )
+static gboolean bee_irc_user_msg( bee_t *bee, bee_user_t *bu, const char *msg_, time_t sent_at )
 {
 	irc_t *irc = bee->ui_data;
 	irc_user_t *iu = (irc_user_t *) bu->ui_data;
 	char *dst, *prefix = NULL;
 	char *wrapped, *ts = NULL;
 	irc_channel_t *ic = NULL;
+	char *msg = g_strdup( msg_ );
+	GSList *l;
 	
 	if( sent_at > 0 && set_getbool( &irc->b->set, "display_timestamps" ) )
 		ts = irc_format_timestamp( irc, sent_at );
@@ -219,8 +210,8 @@ static gboolean bee_irc_user_msg( bee_t *bee, bee_user_t *bu, const char *msg, t
 	{
 		if( iu->last_channel->flags & IRC_CHANNEL_JOINED )
 			ic = iu->last_channel;
-		else if( irc->default_channel->flags & IRC_CHANNEL_JOINED )
-			ic = irc->default_channel;
+		else
+			ic = irc_channel_with_user( irc, iu );
 	}
 	
 	if( ic )
@@ -235,11 +226,41 @@ static gboolean bee_irc_user_msg( bee_t *bee, bee_user_t *bu, const char *msg, t
 		ts = NULL;
 	}
 	
+	for( l = irc_plugins; l; l = l->next )
+	{
+		irc_plugin_t *p = l->data;
+		if( p->filter_msg_in )
+		{
+			char *s = p->filter_msg_in( iu, msg, 0 );
+			if( s )
+			{
+				if( s != msg )
+					g_free( msg );
+				msg = s;
+			}
+			else
+			{
+				/* Modules can swallow messages. */
+				return TRUE;
+			}
+		}
+	}
+	
+	if( ( g_strcasecmp( set_getstr( &bee->set, "strip_html" ), "always" ) == 0 ) ||
+	    ( ( bu->ic->flags & OPT_DOES_HTML ) && set_getbool( &bee->set, "strip_html" ) ) )
+	{
+		char *s = g_strdup( msg );
+		strip_html( s );
+		g_free( msg );
+		msg = s;
+	}
+	
 	wrapped = word_wrap( msg, 425 );
 	irc_send_msg( iu, "PRIVMSG", dst, wrapped, prefix );
 	
 	g_free( wrapped );
 	g_free( prefix );
+	g_free( msg );
 	g_free( ts );
 	
 	return TRUE;
@@ -263,7 +284,6 @@ static gboolean bee_irc_user_nick_update( irc_user_t *iu );
 static gboolean bee_irc_user_fullname( bee_t *bee, bee_user_t *bu )
 {
 	irc_user_t *iu = (irc_user_t *) bu->ui_data;
-	irc_t *irc = (irc_t *) bee->ui_data;
 	char *s;
 	
 	if( iu->fullname != iu->nick )
@@ -277,8 +297,11 @@ static gboolean bee_irc_user_fullname( bee_t *bee, bee_user_t *bu )
 	
 	if( ( bu->ic->flags & OPT_LOGGED_IN ) && set_getbool( &bee->set, "display_namechanges" ) )
 	{
+		/* People don't like this /NOTICE. Meh, let's go back to the old one.
 		char *msg = g_strdup_printf( "<< \002BitlBee\002 - Changed name to `%s' >>", iu->fullname );
 		irc_send_msg( iu, "NOTICE", irc->user->nick, msg, NULL );
+		*/
+		imcb_log( bu->ic, "User `%s' changed name to `%s'", iu->nick, iu->fullname );
 	}
 	
 	bee_irc_user_nick_update( iu );
@@ -358,18 +381,18 @@ static gboolean bee_irc_user_privmsg( irc_user_t *iu, const char *msg )
 			set_getint( &iu->irc->b->set, "away_reply_timeout" );
 	}
 	
+	if( iu->pastebuf == NULL )
+		iu->pastebuf = g_string_new( msg );
+	else
+	{
+		b_event_remove( iu->pastebuf_timer );
+		g_string_append_printf( iu->pastebuf, "\n%s", msg );
+	}
+	
 	if( set_getbool( &iu->irc->b->set, "paste_buffer" ) )
 	{
 		int delay;
 		
-		if( iu->pastebuf == NULL )
-			iu->pastebuf = g_string_new( msg );
-		else
-		{
-			b_event_remove( iu->pastebuf_timer );
-			g_string_append_printf( iu->pastebuf, "\n%s", msg );
-		}
-		
 		if( ( delay = set_getint( &iu->irc->b->set, "paste_buffer_delay" ) ) <= 5 )
 			delay *= 1000;
 		
@@ -378,17 +401,45 @@ static gboolean bee_irc_user_privmsg( irc_user_t *iu, const char *msg )
 		return TRUE;
 	}
 	else
-		return bee_user_msg( iu->irc->b, iu->bu, msg, 0 );
+	{
+		bee_irc_user_privmsg_cb( iu, 0, 0 );
+		
+		return TRUE;
+	}
 }
 
 static gboolean bee_irc_user_privmsg_cb( gpointer data, gint fd, b_input_condition cond )
 {
 	irc_user_t *iu = data;
+	char *msg = g_string_free( iu->pastebuf, FALSE );
+	GSList *l;
 	
-	bee_user_msg( iu->irc->b, iu->bu, iu->pastebuf->str, 0 );
+	for( l = irc_plugins; l; l = l->next )
+	{
+		irc_plugin_t *p = l->data;
+		if( p->filter_msg_out )
+		{
+			char *s = p->filter_msg_out( iu, msg, 0 );
+			if( s )
+			{
+				if( s != msg )
+					g_free( msg );
+				msg = s;
+			}
+			else
+			{
+				/* Modules can swallow messages. */
+				iu->pastebuf = NULL;
+				g_free( msg );
+				return FALSE;
+			}
+		}
+	}
 	
-	g_string_free( iu->pastebuf, TRUE );
-	iu->pastebuf = 0;
+	bee_user_msg( iu->irc->b, iu->bu, msg, 0 );
+	
+	g_free( msg );
+	iu->pastebuf = NULL;
 	iu->pastebuf_timer = 0;
 	
 	return FALSE;
@@ -539,7 +590,7 @@ static gboolean bee_irc_chat_remove_user( bee_t *bee, struct groupchat *c, bee_u
 	irc_t *irc = bee->ui_data;
 	irc_channel_t *ic = c->ui_data;
 	
-	if( ic == NULL )
+	if( ic == NULL || bu == NULL )
 		return FALSE;
 	
 	/* TODO: Possible bug here: If a module removes $user here instead of just
@@ -624,6 +675,63 @@ static gboolean bee_irc_chat_name_hint( bee_t *bee, struct groupchat *c, const c
 	return TRUE;
 }
 
+static gboolean bee_irc_chat_invite( bee_t *bee, bee_user_t *bu, const char *name, const char *msg )
+{
+	char *channel, *s;
+	irc_t *irc = bee->ui_data;
+	irc_user_t *iu = bu->ui_data;
+	irc_channel_t *chan;
+	
+	if( strchr( CTYPES, name[0] ) )
+		channel = g_strdup( name );
+	else
+		channel = g_strdup_printf( "#%s", name );
+	
+	if( ( s = strchr( channel, '@' ) ) )
+		*s = '\0';
+	
+	if( strlen( channel ) > MAX_NICK_LENGTH )
+	{
+		/* If the channel name is very long (like those insane GTalk
+		   UUID names), try if we can use the inviter's nick. */
+		s = g_strdup_printf( "#%s", iu->nick );
+		if( irc_channel_by_name( irc, s ) == NULL )
+		{
+			g_free( channel );
+			channel = s;
+		}
+	}
+	
+	if( ( chan = irc_channel_new( irc, channel ) ) &&
+	    set_setstr( &chan->set, "type", "chat" ) &&
+	    set_setstr( &chan->set, "chat_type", "room" ) &&
+	    set_setstr( &chan->set, "account", bu->ic->acc->tag ) &&
+	    set_setstr( &chan->set, "room", (char*) name ) )
+	{
+		/* I'm assuming that if the user didn't "chat add" the room
+		   himself but got invited, it's temporary, so make this a
+		   temporary mapping that is removed as soon as we /PART. */
+		chan->flags |= IRC_CHANNEL_TEMP;
+	}
+	else
+	{
+		irc_channel_free( chan );
+		chan = NULL;
+	}
+	g_free( channel );
+	
+	irc_send_msg_f( iu, "PRIVMSG", irc->user->nick, "<< \002BitlBee\002 - Invitation to chatroom %s >>", name );
+	if( msg )
+		irc_send_msg( iu, "PRIVMSG", irc->user->nick, msg, NULL );
+	if( chan )
+	{
+		irc_send_msg_f( iu, "PRIVMSG", irc->user->nick, "To join the room, just /join %s", chan->name );
+		irc_send_invite( iu, chan );
+	}
+	
+	return TRUE;
+}
+
 /* IRC->IM */
 static gboolean bee_irc_channel_chat_privmsg_cb( gpointer data, gint fd, b_input_condition cond );
 
@@ -645,7 +753,7 @@ static gboolean bee_irc_channel_chat_privmsg( irc_channel_t *ic, const char *msg
 		if( ( s = strchr( nick, ':' ) ) || ( s = strchr( nick, ',' ) ) )
 		{
 			*s = '\0';
-			if( ( iu = irc_user_by_name( ic->irc, nick ) ) &&
+			if( ( iu = irc_user_by_name( ic->irc, nick ) ) && iu->bu &&
 			    iu->bu->nick && irc_channel_has_user( ic, iu ) )
 			{
 				trans = g_strconcat( iu->bu->nick, msg + ( s - nick ), NULL );
@@ -750,9 +858,9 @@ static gboolean bee_irc_channel_chat_topic( irc_channel_t *ic, const char *new )
 		char *topic = g_strdup( new );
 		c->ic->acc->prpl->chat_topic( c, topic );
 		g_free( topic );
-		return TRUE;
 	}
 		
+	/* Whatever happened, the IM module should ack the topic change. */
 	return FALSE;
 }
 
@@ -920,6 +1028,7 @@ const struct bee_ui_funcs irc_ui_funcs = {
 	bee_irc_chat_remove_user,
 	bee_irc_chat_topic,
 	bee_irc_chat_name_hint,
+	bee_irc_chat_invite,
 	
 	bee_irc_ft_in_start,
 	bee_irc_ft_out_start,
