@@ -43,32 +43,36 @@ void irc_send_login( irc_t *irc )
 	irc_send_num( irc,   2, ":Host %s is running BitlBee " BITLBEE_VERSION " " ARCH "/" CPU ".", irc->root->host );
 	irc_send_num( irc,   3, ":%s", IRCD_INFO );
 	irc_send_num( irc,   4, "%s %s %s %s", irc->root->host, BITLBEE_VERSION, UMODES UMODES_PRIV, CMODES );
-	irc_send_num( irc,   5, "PREFIX=(ov)@+ CHANTYPES=%s CHANMODES=,,,%s NICKLEN=%d NETWORK=BitlBee "
-	                        "CASEMAPPING=rfc1459 MAXTARGETS=1 WATCH=128 :are supported by this server",
-	                        CTYPES, CMODES, MAX_NICK_LENGTH - 1 );
+	irc_send_num( irc,   5, "PREFIX=(ohv)@%%+ CHANTYPES=%s CHANMODES=,,,%s NICKLEN=%d CHANNELLEN=%d "
+	                        "NETWORK=BitlBee SAFELIST CASEMAPPING=rfc1459 MAXTARGETS=1 WATCH=128 "
+	                        ":are supported by this server",
+	                        CTYPES, CMODES, MAX_NICK_LENGTH - 1, MAX_NICK_LENGTH - 1 );
 	irc_send_motd( irc );
 }
 
 void irc_send_motd( irc_t *irc )
 {
+	char motd[2048];
+	size_t len;
 	int fd;
 	
 	fd = open( global.conf->motdfile, O_RDONLY );
-	if( fd == -1 )
+	if( fd == -1 || ( len = read( fd, motd, sizeof( motd ) - 1 ) ) <= 0 )
 	{
 		irc_send_num( irc, 422, ":We don't need MOTDs." );
 	}
 	else
 	{
-		char linebuf[80];	/* Max. line length for MOTD's is 79 chars. It's what most IRC networks seem to do. */
-		char *add, max;
-		int len;
+		char linebuf[80];
+		char *add = "", max, *in;
 		
+		in = motd;
+		motd[len] = '\0';
 		linebuf[79] = len = 0;
 		max = sizeof( linebuf ) - 1;
 		
 		irc_send_num( irc, 375, ":- %s Message Of The Day - ", irc->root->host );
-		while( read( fd, linebuf + len, 1 ) == 1 )
+		while( ( linebuf[len] = *(in++) ) )
 		{
 			if( linebuf[len] == '\n' || len == max )
 			{
@@ -78,13 +82,15 @@ void irc_send_motd( irc_t *irc )
 			}
 			else if( linebuf[len] == '%' )
 			{
-				read( fd, linebuf + len, 1 );
+				linebuf[len] = *(in++);
 				if( linebuf[len] == 'h' )
 					add = irc->root->host;
 				else if( linebuf[len] == 'v' )
 					add = BITLBEE_VERSION;
 				else if( linebuf[len] == 'n' )
 					add = irc->user->nick;
+				else if( linebuf[len] == '\0' )
+					in --;
 				else
 					add = "%";
 				
@@ -97,15 +103,17 @@ void irc_send_motd( irc_t *irc )
 			}
 		}
 		irc_send_num( irc, 376, ":End of MOTD" );
-		close( fd );
 	}
+	
+	if( fd != -1 )
+		close( fd );
 }
 
 void irc_usermsg( irc_t *irc, char *format, ... )
 {
 	irc_channel_t *ic = NULL;
 	irc_user_t *iu = irc->root;
-	char text[1024];
+	char text[2048];
 	va_list params;
 	char *dst;
 	
@@ -118,8 +126,8 @@ void irc_usermsg( irc_t *irc, char *format, ... )
 	{
 		if( iu->last_channel->flags & IRC_CHANNEL_JOINED )
 			ic = iu->last_channel;
-		else if( irc->default_channel->flags & IRC_CHANNEL_JOINED )
-			ic = irc->default_channel;
+		else
+			ic = irc_channel_with_user( irc, irc->root );
 	}
 	
 	if( ic )
@@ -262,7 +270,7 @@ void irc_send_whois( irc_user_t *iu )
 
 void irc_send_who( irc_t *irc, GSList *l, const char *channel )
 {
-	gboolean is_channel = strcmp( channel, "**" ) != 0;
+	gboolean is_channel = strchr( CTYPES, channel[0] ) != NULL;
 	
 	while( l )
 	{
@@ -271,7 +279,7 @@ void irc_send_who( irc_t *irc, GSList *l, const char *channel )
 			iu = ((irc_channel_user_t*)iu)->iu;
 		/* TODO(wilmer): Restore away/channel information here */
 		irc_send_num( irc, 352, "%s %s %s %s %s %c :0 %s",
-		              channel ? : "*", iu->user, iu->host, irc->root->host,
+		              is_channel ? channel : "*", iu->user, iu->host, irc->root->host,
 		              iu->nick, iu->flags & IRC_USER_AWAY ? 'G' : 'H',
 		              iu->fullname );
 		l = l->next;
@@ -325,7 +333,7 @@ void irc_send_msg( irc_user_t *iu, const char *type, const char *dst, const char
 void irc_send_msg_raw( irc_user_t *iu, const char *type, const char *dst, const char *msg )
 {
 	irc_write( iu->irc, ":%s!%s@%s %s %s :%s",
-	           iu->nick, iu->user, iu->host, type, dst, msg );
+	           iu->nick, iu->user, iu->host, type, dst, msg && *msg ? msg : " " );
 }
 
 void irc_send_msg_f( irc_user_t *iu, const char *type, const char *dst, const char *format, ... )
@@ -396,3 +404,11 @@ void irc_send_channel_user_mode_diff( irc_channel_t *ic, irc_user_t *iu,
 	if( *changes )
 		irc_write( ic->irc, ":%s MODE %s %s", from, ic->name, changes );
 }
+
+void irc_send_invite( irc_user_t *iu, irc_channel_t *ic )
+{
+	irc_t *irc = iu->irc;
+	
+	irc_write( iu->irc, ":%s!%s@%s INVITE %s :%s",
+	           iu->nick, iu->user, iu->host, irc->user->nick, ic->name );
+}
