@@ -24,6 +24,7 @@
 */
 
 #include <gnutls/gnutls.h>
+#include <gcrypt.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include "proxy.h"
@@ -60,6 +61,16 @@ static gboolean ssl_starttls_real( gpointer data, gint source, b_input_condition
 static gboolean ssl_handshake( gpointer data, gint source, b_input_condition cond );
 
 
+void ssl_init( void )
+{
+	if( initialized )
+		return;
+	
+	gnutls_global_init();
+	initialized = TRUE;
+	atexit( gnutls_global_deinit );
+}
+
 void *ssl_connect( char *host, int port, ssl_input_function func, gpointer data )
 {
 	struct scd *conn = g_new0( struct scd, 1 );
@@ -119,12 +130,7 @@ static gboolean ssl_connected( gpointer data, gint source, b_input_condition con
 		return FALSE;
 	}
 	
-	if( !initialized )
-	{
-		gnutls_global_init();
-		initialized = TRUE;
-		atexit( gnutls_global_deinit );
-	}
+	ssl_init();
 	
 	gnutls_certificate_allocate_credentials( &conn->xcred );
 	gnutls_init( &conn->session, GNUTLS_CLIENT );
@@ -188,6 +194,8 @@ int ssl_read( void *conn, char *buf, int len )
 	if( st == GNUTLS_E_AGAIN || st == GNUTLS_E_INTERRUPTED )
 		ssl_errno = SSL_AGAIN;
 	
+	if( 0 && getenv( "BITLBEE_DEBUG" ) && st > 0 ) len = write( 1, buf, st );
+	
 	return st;
 }
 
@@ -207,6 +215,8 @@ int ssl_write( void *conn, const char *buf, int len )
 	if( st == GNUTLS_E_AGAIN || st == GNUTLS_E_INTERRUPTED )
 		ssl_errno = SSL_AGAIN;
 	
+	if( 0 && getenv( "BITLBEE_DEBUG" ) && st > 0 ) len = write( 1, buf, st );
+	
 	return st;
 }
 
@@ -245,3 +255,26 @@ b_input_condition ssl_getdirection( void *conn )
 	return( gnutls_record_get_direction( ((struct scd*)conn)->session ) ?
 	        B_EV_IO_WRITE : B_EV_IO_READ );
 }
+
+size_t ssl_des3_encrypt( const unsigned char *key, size_t key_len, const unsigned char *input,
+                         size_t input_len, const unsigned char *iv, unsigned char **res )
+{
+	gcry_cipher_hd_t gcr;
+	gcry_error_t st;
+	
+	ssl_init();
+	
+	*res = g_malloc( input_len  );
+	st = gcry_cipher_open( &gcr, GCRY_CIPHER_3DES, GCRY_CIPHER_MODE_CBC, 0 ) ||
+	     gcry_cipher_setkey( gcr, key, key_len ) ||
+	     gcry_cipher_setiv( gcr, iv, 8 ) ||
+	     gcry_cipher_encrypt( gcr, *res, input_len, input, input_len );
+	
+	gcry_cipher_close( gcr );
+	
+	if( st == 0 )
+		return input_len;
+	
+	g_free( *res );
+	return 0;
+}
