@@ -95,7 +95,7 @@ static void jabber_login( account_t *acc )
 {
 	struct im_connection *ic = imcb_new( acc );
 	struct jabber_data *jd = g_new0( struct jabber_data, 1 );
-	struct ns_srv_reply *srv = NULL;
+	struct ns_srv_reply **srvl = NULL, *srv = NULL;
 	char *connect_to, *s;
 	int i;
 	
@@ -195,9 +195,19 @@ static void jabber_login( account_t *acc )
 	/* Figure out the hostname to connect to. */
 	if( acc->server && *acc->server )
 		connect_to = acc->server;
-	else if( ( srv = srv_lookup( "xmpp-client", "tcp", jd->server ) ) ||
-		 ( srv = srv_lookup( "jabber-client", "tcp", jd->server ) ) )
+	else if( ( srvl = srv_lookup( "xmpp-client", "tcp", jd->server ) ) ||
+	         ( srvl = srv_lookup( "jabber-client", "tcp", jd->server ) ) )
+	{
+		/* Find the lowest-priority one. These usually come
+		   back in random/shuffled order. Not looking at
+		   weights etc for now. */
+		srv = *srvl;
+		for( i = 1; srvl[i]; i ++ )
+			if( srvl[i]->prio < srv->prio )
+				srv = srvl[i];
+		
 		connect_to = srv->name;
+	}
 	else
 		connect_to = jd->server;
 	
@@ -226,7 +236,7 @@ static void jabber_login( account_t *acc )
 	{
 		jd->fd = proxy_connect( connect_to, srv ? srv->port : set_getint( &acc->set, "port" ), jabber_connected_plain, ic );
 	}
-	g_free( srv );
+	srv_free( srvl );
 	
 	if( jd->fd == -1 )
 	{
@@ -380,7 +390,7 @@ static void jabber_get_info( struct im_connection *ic, char *who )
 		imcb_log( ic, "Buddy %s (%d) information:", bud->full_jid, bud->priority );
 		if( bud->away_state )
 			imcb_log( ic, "Away state: %s", bud->away_state->full_name );
-		imcb_log( ic, "Status message: %s", bud->away_message ? : "(none)" );
+		imcb_log( ic, "Status message: %s", bud->away_message ? bud->away_message : "(none)" );
 		
 		bud = bud->next;
 	}
@@ -394,8 +404,10 @@ static void jabber_set_away( struct im_connection *ic, char *state_txt, char *me
 	
 	/* state_txt == NULL -> Not away.
 	   Unknown state -> fall back to the first defined away state. */
-	jd->away_state = state_txt ? jabber_away_state_by_name( state_txt )
-	                 ? : jabber_away_state_list : NULL;
+	if( state_txt == NULL )
+		jd->away_state = NULL;
+	else if( ( jd->away_state = jabber_away_state_by_name( state_txt ) ) == NULL )
+		jd->away_state = jabber_away_state_list;
 	
 	g_free( jd->away_message );
 	jd->away_message = ( message && *message ) ? g_strdup( message ) : NULL;
@@ -447,7 +459,7 @@ static struct groupchat *jabber_chat_join_( struct im_connection *ic, const char
 	else if( jabber_chat_by_jid( ic, room ) )
 		imcb_error( ic, "Already present in chat `%s'", room );
 	else
-		return jabber_chat_join( ic, room, nick, password );
+		return jabber_chat_join( ic, room, nick, set_getstr( sets, "password" ) );
 	
 	return NULL;
 }
@@ -539,11 +551,24 @@ static int jabber_send_typing( struct im_connection *ic, char *who, int typing )
 	return 1;
 }
 
+void jabber_chat_add_settings( account_t *acc, set_t **head )
+{
+	/* Meh. Stupid room passwords. Not trying to obfuscate/hide
+	   them from the user for now. */
+	set_add( head, "password", NULL, NULL, NULL );
+}
+
+void jabber_chat_free_settings( account_t *acc, set_t **head )
+{
+	set_del( head, "password" );
+}
+
 void jabber_initmodule()
 {
 	struct prpl *ret = g_new0( struct prpl, 1 );
 	
 	ret->name = "jabber";
+	ret->mms = 0;                        /* no limit */
 	ret->login = jabber_login;
 	ret->init = jabber_init;
 	ret->logout = jabber_logout;
@@ -559,6 +584,8 @@ void jabber_initmodule()
 	ret->chat_invite = jabber_chat_invite_;
 	ret->chat_leave = jabber_chat_leave_;
 	ret->chat_join = jabber_chat_join_;
+	ret->chat_add_settings = jabber_chat_add_settings;
+	ret->chat_free_settings = jabber_chat_free_settings;
 	ret->keepalive = jabber_keepalive;
 	ret->send_typing = jabber_send_typing;
 	ret->handle_cmp = g_strcasecmp;
