@@ -79,6 +79,28 @@ static void purple_init( account_t *acc )
 	set_t *s;
 	char help_title[64];
 	GString *help;
+	static gboolean dir_fixed = FALSE;
+	
+	/* Layer violation coming up: Making an exception for libpurple here.
+	   Dig in the IRC state a bit to get a username. Ideally we should
+	   check if s/he identified but this info doesn't seem *that* important.
+	   It's just that fecking libpurple can't *not* store this shit.
+	   
+	   Remember that libpurple is not really meant to be used on public
+	   servers anyway! */
+	if( !dir_fixed )
+	{
+		irc_t *irc = acc->bee->ui_data;
+		char *dir;
+		
+		dir = g_strdup_printf( "%s/purple/%s", global.conf->configdir, irc->user->nick );
+		purple_util_set_user_dir( dir );
+		g_free( dir );
+		
+		purple_blist_load();
+		purple_prefs_load();
+		dir_fixed = TRUE;
+	}
 	
 	help = g_string_new( "" );
 	g_string_printf( help, "BitlBee libpurple module %s (%s).\n\nSupported settings:",
@@ -186,6 +208,9 @@ static void purple_init( account_t *acc )
 		s->flags |= ACC_SET_OFFLINE_ONLY;
 	}
 	
+	if( strcmp( prpl->info->name, "Gadu-Gadu" ) == 0 )
+		s = set_add( &acc->set, "gg_sync_contacts", "true", set_eval_bool, acc );
+	
 	/* Go through all away states to figure out if away/status messages
 	   are possible. */
 	pa = purple_account_new( acc->user, (char*) acc->prpl->data );
@@ -253,7 +278,8 @@ static void purple_login( account_t *acc )
 	struct im_connection *ic = imcb_new( acc );
 	PurpleAccount *pa;
 	
-	if( local_bee != NULL && local_bee != acc->bee )
+	if( ( local_bee != NULL && local_bee != acc->bee ) ||
+	    ( global.conf->runmode == RUNMODE_DAEMON && !getenv( "BITLBEE_DEBUG" ) ) )
 	{
 		imcb_error( ic,  "Daemon mode detected. Do *not* try to use libpurple in daemon mode! "
 		                 "Please use inetd or ForkDaemon mode instead." );
@@ -351,9 +377,63 @@ static char *set_eval_display_name( set_t *set, char *value )
 	account_t *acc = set->data;
 	struct im_connection *ic = acc->ic;
 	
+	if( ic )
+		imcb_log( ic, "Changing display_name not currently supported with libpurple!" );
+	
 	return NULL;
 }
 
+/* Bad bad gadu-gadu, not saving buddy list by itself */
+static void purple_gg_buddylist_export( PurpleConnection *gc )
+{
+	struct im_connection *ic = purple_ic_by_gc( gc );
+	
+	if( set_getstr( &ic->acc->set, "gg_sync_contacts" ) )
+	{
+		GList *actions = gc->prpl->info->actions( gc->prpl, gc );
+		GList *p;
+		for( p = g_list_first(actions); p; p = p->next )
+		{
+			if( ((PurplePluginAction*)p->data) &&
+			    purple_menu_cmp( ((PurplePluginAction*)p->data)->label, "Upload buddylist to Server" ) == 0)
+			{
+				PurplePluginAction action;
+				action.plugin = gc->prpl;
+				action.context = gc;
+				action.user_data = NULL;
+				((PurplePluginAction*)p->data)->callback(&action);
+				break;
+			}
+		}
+		g_list_free( actions );
+	}
+}
+
+static void purple_gg_buddylist_import( PurpleConnection *gc )
+{
+	struct im_connection *ic = purple_ic_by_gc( gc );
+	
+	if( set_getstr( &ic->acc->set, "gg_sync_contacts" ) )
+	{
+		GList *actions = gc->prpl->info->actions( gc->prpl, gc );
+		GList *p;
+		for( p = g_list_first(actions); p; p = p->next )
+		{
+			if( ((PurplePluginAction*)p->data) &&
+			    purple_menu_cmp( ((PurplePluginAction*)p->data)->label, "Download buddylist from Server" ) == 0 )
+			{
+				PurplePluginAction action;
+				action.plugin = gc->prpl;
+				action.context = gc;
+				action.user_data = NULL;
+				((PurplePluginAction*)p->data)->callback(&action);
+				break;
+			}
+		}
+		g_list_free( actions );
+	}
+}
+
 static void purple_add_buddy( struct im_connection *ic, char *who, char *group )
 {
 	PurpleBuddy *pb;
@@ -368,6 +448,8 @@ static void purple_add_buddy( struct im_connection *ic, char *who, char *group )
 	pb = purple_buddy_new( (PurpleAccount*) ic->proto_data, who, NULL );
 	purple_blist_add_buddy( pb, NULL, pg, NULL );
 	purple_account_add_buddy( (PurpleAccount*) ic->proto_data, pb );
+
+	purple_gg_buddylist_export( ((PurpleAccount*)ic->proto_data)->gc );
 }
 
 static void purple_remove_buddy( struct im_connection *ic, char *who, char *group )
@@ -384,6 +466,8 @@ static void purple_remove_buddy( struct im_connection *ic, char *who, char *grou
 		
 		purple_blist_remove_buddy( pb );
 	}
+
+	purple_gg_buddylist_export( ((PurpleAccount*)ic->proto_data)->gc );
 }
 
 static void purple_add_permit( struct im_connection *ic, char *who )
@@ -426,23 +510,16 @@ static void purple_keepalive( struct im_connection *ic )
 static int purple_send_typing( struct im_connection *ic, char *who, int flags )
 {
 	PurpleTypingState state = PURPLE_NOT_TYPING;
-	PurpleConversation *conv;
+	PurpleAccount *pa = ic->proto_data;
 	
 	if( flags & OPT_TYPING )
 		state = PURPLE_TYPING;
 	else if( flags & OPT_THINKING )
 		state = PURPLE_TYPED;
 	
-	if( ( conv = purple_find_conversation_with_account( PURPLE_CONV_TYPE_IM,
-	                                                    who, ic->proto_data ) ) == NULL )
-	{
-		purple_conv_im_set_typing_state( purple_conversation_get_im_data( conv ), state );
-		return 1;
-	}
-	else
-	{
-		return 0;
-	}
+	serv_send_typing( purple_account_get_connection( pa ), who, state );
+	
+	return 1;
 }
 
 static void purple_chat_msg( struct groupchat *gc, char *message, int flags )
@@ -523,7 +600,7 @@ void purple_chat_leave( struct groupchat *gc )
 	purple_conversation_destroy( pc );
 }
 
-struct groupchat *purple_chat_join( struct im_connection *ic, const char *room, const char *nick, const char *password )
+struct groupchat *purple_chat_join( struct im_connection *ic, const char *room, const char *nick, const char *password, set_t **sets )
 {
 	PurpleAccount *pa = ic->proto_data;
 	PurplePlugin *prpl = purple_plugins_find_with_id( pa->protocol_id );
@@ -609,6 +686,9 @@ static void prplcb_conn_connected( PurpleConnection *gc )
 		g_free( s->value );
 		s->value = g_strdup( dn );
 	}
+
+	// user list needs to be requested for Gadu-Gadu
+	purple_gg_buddylist_import( gc );
 	
 	if( gc->flags & PURPLE_CONNECTION_HTML )
 		ic->flags |= OPT_DOES_HTML;
@@ -808,6 +888,32 @@ static void prplcb_conv_im( PurpleConversation *conv, const char *who, const cha
 	imcb_buddy_msg( ic, (char*) who, (char*) message, 0, mtime );
 }
 
+/* No, this is not a ui_op but a signal. */
+static void prplcb_buddy_typing( PurpleAccount *account, const char *who, gpointer null )
+{
+	PurpleConversation *conv;
+	PurpleConvIm *im;
+	int state;
+	
+	if( ( conv = purple_find_conversation_with_account( PURPLE_CONV_TYPE_IM, who, account ) ) == NULL )
+		return;
+	
+	im = PURPLE_CONV_IM(conv);
+	switch( purple_conv_im_get_typing_state( im ) )
+	{
+	case PURPLE_TYPING:
+		state = OPT_TYPING;
+		break;
+	case PURPLE_TYPED:
+		state = OPT_THINKING;
+		break;
+	default:
+		state = 0;
+	}
+	
+	imcb_buddy_typing( purple_ic_by_pa( account ), who, state );
+}
+
 static PurpleConversationUiOps bee_conv_uiops = 
 {
 	prplcb_conv_new,           /* create_conversation  */
@@ -1112,6 +1218,7 @@ void purple_initmodule()
 	struct prpl funcs;
 	GList *prots;
 	GString *help;
+	char *dir;
 	
 	if( B_EV_IO_READ != PURPLE_INPUT_READ ||
 	    B_EV_IO_WRITE != PURPLE_INPUT_WRITE )
@@ -1120,7 +1227,10 @@ void purple_initmodule()
 		exit( 1 );
 	}
 	
-	purple_util_set_user_dir( "/tmp" );
+	dir = g_strdup_printf( "%s/purple", global.conf->configdir );
+	purple_util_set_user_dir( dir );
+	g_free( dir );
+	
 	purple_debug_set_enabled( FALSE );
 	purple_core_set_ui_ops( &bee_core_uiops );
 	purple_eventloop_set_ui_ops( &glib_eventloops );
@@ -1131,12 +1241,16 @@ void purple_initmodule()
 		abort();
 	}
 	
-	/* This seems like stateful shit we don't want... */
 	purple_set_blist( purple_blist_new() );
-	purple_blist_load();
 	
-	/* Meh? */
-	purple_prefs_load();
+	/* No, really. So far there were ui_ops for everything, but now suddenly
+	   one needs to use signals for typing notification stuff. :-( */
+	purple_signal_connect( purple_conversations_get_handle(), "buddy-typing",
+	                       &funcs, PURPLE_CALLBACK(prplcb_buddy_typing), NULL );
+	purple_signal_connect( purple_conversations_get_handle(), "buddy-typed",
+	                       &funcs, PURPLE_CALLBACK(prplcb_buddy_typing), NULL );
+	purple_signal_connect( purple_conversations_get_handle(), "buddy-typing-stopped",
+	                       &funcs, PURPLE_CALLBACK(prplcb_buddy_typing), NULL );
 	
 	memset( &funcs, 0, sizeof( funcs ) );
 	funcs.login = purple_login;
@@ -1192,7 +1306,8 @@ void purple_initmodule()
 	}
 	
 	g_string_append( help, "\n\nFor used protocols, more information about available "
-	                 "settings can be found using \x02help purple <protocol name>\x02" );
+	                 "settings can be found using \x02help purple <protocol name>\x02 "
+	                 "(create an account using that protocol first!)" );
 	
 	/* Add a simple dynamically-generated help item listing all
 	   the supported protocols. */
