@@ -28,6 +28,17 @@
 #include "twitter_lib.h"
 #include "url.h"
 
+#define twitter_msg( ic, fmt... ) \
+	do {                                                        \
+		struct twitter_data *td = ic->proto_data;           \
+		if( td->home_timeline_gc )                          \
+			imcb_chat_log( td->home_timeline_gc, fmt ); \
+		else                                                \
+			imcb_log( ic, fmt );                        \
+	} while( 0 );
+		
+GSList *twitter_connections = NULL;
+
 /**
  * Main loop function
  */
@@ -96,6 +107,10 @@ static void twitter_oauth_start( struct im_connection *ic )
 	imcb_log( ic, "Requesting OAuth request token" );
 
 	td->oauth_info = oauth_request_token( &twitter_oauth, twitter_oauth_callback, ic );
+	
+	/* We need help from the user to complete OAuth login, so don't time
+	   out on this login. */
+	ic->flags |= OPT_SLOW_LOGIN;
 }
 
 static gboolean twitter_oauth_callback( struct oauth_info *info )
@@ -185,12 +200,16 @@ static void twitter_init( account_t *acc )
 		def_oauth = "false";
 	}
 	
+	s = set_add( &acc->set, "auto_reply_timeout", "10800", set_eval_int, acc );
+	
 	s = set_add( &acc->set, "base_url", def_url, NULL, acc );
 	s->flags |= ACC_SET_OFFLINE_ONLY;
 	
+	s = set_add( &acc->set, "commands", "true", set_eval_bool, acc );
+	
 	s = set_add( &acc->set, "message_length", "140", set_eval_int, acc );
 	
-	s = set_add( &acc->set, "mode", "one", set_eval_mode, acc );
+	s = set_add( &acc->set, "mode", "chat", set_eval_mode, acc );
 	s->flags |= ACC_SET_OFFLINE_ONLY;
 	
 	s = set_add( &acc->set, "oauth", def_oauth, set_eval_bool, acc );
@@ -252,7 +271,7 @@ static void twitter_logout( struct im_connection *ic )
 	struct twitter_data *td = ic->proto_data;
 	
 	// Set the status to logged out.
-	ic->flags = 0;
+	ic->flags &= ~ OPT_LOGGED_IN;
 
 	// Remove the main_loop function from the function queue.
 	b_event_remove(td->main_loop_id);
@@ -273,6 +292,8 @@ static void twitter_logout( struct im_connection *ic )
 	twitter_connections = g_slist_remove( twitter_connections, ic );
 }
 
+static void twitter_handle_command( struct im_connection *ic, char *message );
+
 /**
  *
  */
@@ -301,8 +322,8 @@ static int twitter_buddy_msg( struct im_connection *ic, char *who, char *message
 				return FALSE;
 			}
 		}
-		else if( twitter_length_check(ic, message) )
-			twitter_post_status(ic, message);
+		else
+			twitter_handle_command(ic, message);
 	}
 	else
 	{
@@ -334,28 +355,8 @@ static void twitter_remove_buddy( struct im_connection *ic, char *who, char *gro
 
 static void twitter_chat_msg( struct groupchat *c, char *message, int flags )
 {
-	if( c && message && twitter_length_check( c->ic, message ) )
-	{
-		char *s, *new = NULL;
-		
-		if( ( s = strchr( message, ':' ) ) ||
-		    ( s = strchr( message, ',' ) ) )
-		{
-			bee_user_t *bu;
-			
-			new = g_strdup( message );
-			new[s-message] = '\0';
-			if( ( bu = bee_user_by_handle( c->ic->bee, c->ic, new ) ) )
-			{
-				sprintf( new, "@%s", bu->handle );
-				new[s-message+1] = ' ';
-				message = new;
-			}
-		}
-		
-		twitter_post_status( c->ic, message );
-		g_free( new );
-	}
+	if( c && message )
+		twitter_handle_command( c->ic, message );
 }
 
 static void twitter_chat_invite( struct groupchat *c, char *who, char *message )
@@ -410,10 +411,134 @@ static int twitter_send_typing( struct im_connection *ic, char *who, int typing
 //	return value;
 //}
 
+static void twitter_buddy_data_add( struct bee_user *bu )
+{
+	bu->data = g_new0( struct twitter_user_data, 1 );
+}
+
+static void twitter_buddy_data_free( struct bee_user *bu )
+{
+	g_free( bu->data );
+}
+
+static void twitter_handle_command( struct im_connection *ic, char *message )
+{
+	struct twitter_data *td = ic->proto_data;
+	char *cmds, **cmd;
+	
+	cmds = g_strdup( message );
+	cmd = split_command_parts( cmds );
+	
+	if( cmd[0] == NULL )
+	{
+		g_free( cmds );
+		return;
+	}
+	else if( !set_getbool( &ic->acc->set, "commands" ) )
+	{
+		/* Not supporting commands. */
+	}
+	else if( g_strcasecmp( cmd[0], "undo" ) == 0 )
+	{
+		guint64 id;
+		
+		if( cmd[1] )
+			id = g_ascii_strtoull( cmd[1], NULL, 10 );
+		else
+			id = td->last_status_id;
+		
+		/* TODO: User feedback. */
+		if( id )
+			twitter_status_destroy( ic, id );
+		else
+			twitter_msg( ic, "Could not undo last action" );
+		
+		g_free( cmds );
+		return;
+	}
+	else if( g_strcasecmp( cmd[0], "follow" ) == 0 && cmd[1] )
+	{
+		twitter_add_buddy( ic, cmd[1], NULL );
+		g_free( cmds );
+		return;
+	}
+	else if( g_strcasecmp( cmd[0], "unfollow" ) == 0 && cmd[1] )
+	{
+		twitter_remove_buddy( ic, cmd[1], NULL );
+		g_free( cmds );
+		return;
+	}
+	else if( g_strcasecmp( cmd[0], "rt" ) == 0 && cmd[1] )
+	{
+		struct twitter_user_data *tud;
+		bee_user_t *bu;
+		guint64 id;
+		
+		if( ( bu = bee_user_by_handle( ic->bee, ic, cmd[1] ) ) &&
+		    ( tud = bu->data ) && tud->last_id )
+			id = tud->last_id;
+		else
+			id = g_ascii_strtoull( cmd[1], NULL, 10 );
+		
+		td->last_status_id = 0;
+		if( id )
+			twitter_status_retweet( ic, id );
+		else
+			twitter_msg( ic, "User `%s' does not exist or didn't "
+			                 "post any statuses recently", cmd[1] );
+		
+		g_free( cmds );
+		return;
+	}
+	else if( g_strcasecmp( cmd[0], "post" ) == 0 )
+	{
+		message += 5;
+	}
+	
+	{
+		guint64 in_reply_to = 0;
+		char *s, *new = NULL;
+		bee_user_t *bu;
+		
+		if( !twitter_length_check( ic, message ) )
+		{
+			g_free( cmds );
+		  	return;
+		}
+		
+		s = cmd[0] + strlen( cmd[0] ) - 1;
+		if( s > cmd[0] && ( *s == ':' || *s == ',' ) )
+		{
+			*s = '\0';
+			
+			if( ( bu = bee_user_by_handle( ic->bee, ic, cmd[0] ) ) )
+			{
+				struct twitter_user_data *tud = bu->data;
+				
+				new = g_strdup_printf( "@%s %s", bu->handle,
+				                       message + ( s - cmd[0] ) + 2 );
+				message = new;
+				
+				if( time( NULL ) < tud->last_time +
+				    set_getint( &ic->acc->set, "auto_reply_timeout" ) )
+					in_reply_to = tud->last_id;
+			}
+		}
+		
+		/* If the user runs undo between this request and its response
+		   this would delete the second-last Tweet. Prevent that. */
+		td->last_status_id = 0;
+		twitter_post_status( ic, message, in_reply_to );
+		g_free( new );
+	}
+	g_free( cmds );
+}
+
 void twitter_initmodule()
 {
 	struct prpl *ret = g_new0(struct prpl, 1);
 	
+	ret->options = OPT_NOOTR;
 	ret->name = "twitter";
 	ret->login = twitter_login;
 	ret->init = twitter_init;
@@ -433,6 +558,8 @@ void twitter_initmodule()
 	ret->add_deny = twitter_add_deny;
 	ret->rem_deny = twitter_rem_deny;
 	ret->send_typing = twitter_send_typing;
+	ret->buddy_data_add = twitter_buddy_data_add;
+	ret->buddy_data_free = twitter_buddy_data_free;
 	ret->handle_cmp = g_strcasecmp;
 	
 	register_protocol(ret);
@@ -441,7 +568,4 @@ void twitter_initmodule()
 	ret = g_memdup(ret, sizeof(struct prpl));
 	ret->name = "identica";
 	register_protocol(ret);
-
-	// Initialise the twitter_connections GSList.
-	twitter_connections = NULL;
 }
