@@ -54,17 +54,17 @@ void root_command( irc_t *irc, char *cmd[] )
 		return;
 	
 	len = strlen( cmd[0] );
-	for( i = 0; commands[i].command; i++ )
-		if( g_strncasecmp( commands[i].command, cmd[0], len ) == 0 )
+	for( i = 0; root_commands[i].command; i++ )
+		if( g_strncasecmp( root_commands[i].command, cmd[0], len ) == 0 )
 		{
-			if( commands[i+1].command &&
-		            g_strncasecmp( commands[i+1].command, cmd[0], len ) == 0 )
+			if( root_commands[i+1].command &&
+		            g_strncasecmp( root_commands[i+1].command, cmd[0], len ) == 0 )
 		        	/* Only match on the first letters if the match is unique. */
 		        	break;
 		        
-			MIN_ARGS( commands[i].required_parameters );
+			MIN_ARGS( root_commands[i].required_parameters );
 			
-			commands[i].execute( irc, cmd );
+			root_commands[i].execute( irc, cmd );
 			return;
 		}
 	
@@ -99,6 +99,7 @@ static void cmd_help( irc_t *irc, char **cmd )
 }
 
 static void cmd_account( irc_t *irc, char **cmd );
+static void bitlbee_whatsnew( irc_t *irc );
 
 static void cmd_identify( irc_t *irc, char **cmd )
 {
@@ -155,6 +156,8 @@ static void cmd_identify( irc_t *irc, char **cmd )
 		irc->status |= USTATUS_IDENTIFIED;
 		irc_umode_set( irc, "+R", 1 );
 		
+		bitlbee_whatsnew( irc );
+		
 		/* The following code is a bit hairy now. With takeover
 		   support, we shouldn't immediately auto_connect in case
 		   we're going to offer taking over an existing session.
@@ -202,6 +205,8 @@ gboolean cmd_identify_finish( gpointer data, gint fd, b_input_condition cond )
 
 static void cmd_register( irc_t *irc, char **cmd )
 {
+	char s[16];
+	
 	if( global.conf->authmode == AUTHMODE_REGISTERED )
 	{
 		irc_usermsg( irc, "This server does not allow registering new accounts" );
@@ -218,6 +223,12 @@ static void cmd_register( irc_t *irc, char **cmd )
 			irc_setpass( irc, cmd[1] );
 			irc->status |= USTATUS_IDENTIFIED;
 			irc_umode_set( irc, "+R", 1 );
+			
+			/* Set this var now, or anyone who logs in to his/her
+			   newly created account for the first time gets the
+			   whatsnew story. */
+			g_snprintf( s, sizeof( s ), "%d", BITLBEE_VERSION_CODE );
+			set_setstr( &irc->b->set, "last_version", s );
 			break;
 
 		default:
@@ -262,10 +273,20 @@ static void cmd_save( irc_t *irc, char **cmd )
 
 static void cmd_showset( irc_t *irc, set_t **head, char *key )
 {
+	set_t *set;
 	char *val;
 	
 	if( ( val = set_getstr( head, key ) ) )
 		irc_usermsg( irc, "%s = `%s'", key, val );
+	else if( !( set = set_find( head, key ) ) )
+	{
+		irc_usermsg( irc, "Setting `%s' does not exist.", key );
+		if( *head == irc->b->set )
+			irc_usermsg( irc, "It might be an account or channel setting. "
+			             "See \x02help account set\x02 and \x02help channel set\x02." );
+	}
+	else if( set->flags & SET_PASSWORD )
+		irc_usermsg( irc, "%s = `********' (hidden)", key );
 	else
 		irc_usermsg( irc, "%s is empty", key );
 }
@@ -303,7 +324,8 @@ static int cmd_set_real( irc_t *irc, char **cmd, set_t **head, cmd_set_checkflag
 		else
 			st = set_setstr( head, set_name, value );
 		
-		if( set_getstr( head, set_name ) == NULL )
+		if( set_getstr( head, set_name ) == NULL &&
+		    set_find( head, set_name ) )
 		{
 			/* This happens when changing the passwd, for example.
 			   Showing these msgs instead gives slightly clearer
@@ -327,7 +349,8 @@ static int cmd_set_real( irc_t *irc, char **cmd, set_t **head, cmd_set_checkflag
 		set_t *s = *head;
 		while( s )
 		{
-			cmd_showset( irc, &s, s->key );
+			if( !( s->flags & SET_HIDDEN ) )
+				cmd_showset( irc, &s, s->key );
 			s = s->next;
 		}
 	}
@@ -370,7 +393,16 @@ static void cmd_account( irc_t *irc, char **cmd )
 	{
 		struct prpl *prpl;
 		
-		MIN_ARGS( 4 );
+		MIN_ARGS( 3 );
+		
+		if( cmd[4] == NULL )
+			for( a = irc->b->accounts; a; a = a->next )
+				if( strcmp( a->pass, PASSWORD_PENDING ) == 0 )
+				{
+					irc_usermsg( irc, "Enter password for account %s(%s) "
+					             "first (use /OPER)", a->prpl->name, a->user );
+					return;
+				}
 		
 		prpl = find_protocol( cmd[2] );
 		
@@ -386,7 +418,7 @@ static void cmd_account( irc_t *irc, char **cmd )
 				             "protocol `%s' and username `%s'. Are you accidentally "
 				             "trying to add it twice?", prpl->name, cmd[3] );
 		
-		a = account_add( irc->b, prpl, cmd[3], cmd[4] );
+		a = account_add( irc->b, prpl, cmd[3], cmd[4] ? cmd[4] : PASSWORD_PENDING );
 		if( cmd[5] )
 		{
 			irc_usermsg( irc, "Warning: Passing a servername/other flags to `account add' "
@@ -394,7 +426,8 @@ static void cmd_account( irc_t *irc, char **cmd )
 			set_setstr( &a->set, "server", cmd[5] );
 		}
 		
-		irc_usermsg( irc, "Account successfully added" );
+		irc_usermsg( irc, "Account successfully added%s", cmd[4] ? "" :
+		             ", now use /OPER to enter the password" );
 		
 		return;
 	}
@@ -438,7 +471,13 @@ static void cmd_account( irc_t *irc, char **cmd )
 		
 			for( a = irc->b->accounts; a; a = a->next )
 				if( !a->ic && a->auto_connect )
-					account_on( irc->b, a );
+				{
+					if( strcmp( a->pass, PASSWORD_PENDING ) == 0 )
+						irc_usermsg( irc, "Enter password for account %s(%s) "
+						             "first (use /OPER)", a->prpl->name, a->user );
+					else
+						account_on( irc->b, a );
+				}
 		} 
 		else
 		{
@@ -496,6 +535,9 @@ static void cmd_account( irc_t *irc, char **cmd )
 	{
 		if( a->ic )
 			irc_usermsg( irc, "Account already online" );
+		else if( strcmp( a->pass, PASSWORD_PENDING ) == 0 )
+			irc_usermsg( irc, "Enter password for account %s(%s) "
+			             "first (use /OPER)", a->prpl->name, a->user );
 		else
 			account_on( irc->b, a );
 	}
@@ -1007,12 +1049,17 @@ static void cmd_blist( irc_t *irc, char **cmd )
 	
 	irc_usermsg( irc, format, "Nick", "Handle/Account", "Status" );
 	
+	if( irc->root->last_channel &&
+	    strcmp( set_getstr( &irc->root->last_channel->set, "type" ), "control" ) != 0 )
+		irc->root->last_channel = NULL;
+	
 	for( l = irc->users; l; l = l->next )
 	{
 		irc_user_t *iu = l->data;
 		bee_user_t *bu = iu->bu;
 		
-		if( !bu || ( bu->flags & ( BEE_USER_ONLINE | BEE_USER_AWAY ) ) != BEE_USER_ONLINE )
+		if( !bu || ( irc->root->last_channel && !irc_channel_wants_user( irc->root->last_channel, iu ) ) ||
+		    ( bu->flags & ( BEE_USER_ONLINE | BEE_USER_AWAY ) ) != BEE_USER_ONLINE )
 			continue;
 		
 		if( online == 1 )
@@ -1034,7 +1081,8 @@ static void cmd_blist( irc_t *irc, char **cmd )
 		irc_user_t *iu = l->data;
 		bee_user_t *bu = iu->bu;
 		
-		if( !bu || !( bu->flags & BEE_USER_ONLINE ) || !( bu->flags & BEE_USER_AWAY ) )
+		if( !bu || ( irc->root->last_channel && !irc_channel_wants_user( irc->root->last_channel, iu ) ) ||
+		    !( bu->flags & BEE_USER_ONLINE ) || !( bu->flags & BEE_USER_AWAY ) )
 			continue;
 		
 		if( away == 1 )
@@ -1050,7 +1098,8 @@ static void cmd_blist( irc_t *irc, char **cmd )
 		irc_user_t *iu = l->data;
 		bee_user_t *bu = iu->bu;
 		
-		if( !bu || bu->flags & BEE_USER_ONLINE )
+		if( !bu || ( irc->root->last_channel && !irc_channel_wants_user( irc->root->last_channel, iu ) ) ||
+		    bu->flags & BEE_USER_ONLINE )
 			continue;
 		
 		if( offline == 1 )
@@ -1264,8 +1313,36 @@ static void cmd_transfer( irc_t *irc, char **cmd )
 	}
 }
 
+static void cmd_nick( irc_t *irc, char **cmd )
+{
+	irc_usermsg( irc, "This command is deprecated. Try: account %s set display_name", cmd[1] );
+}
+
+/* Maybe this should be a stand-alone command as well? */
+static void bitlbee_whatsnew( irc_t *irc )
+{
+	int last = set_getint( &irc->b->set, "last_version" );
+	char s[16], *msg;
+	
+	if( last >= BITLBEE_VERSION_CODE )
+		return;
+	
+	msg = help_get_whatsnew( &(global.help), last );
+	
+	if( msg )
+		irc_usermsg( irc, "%s: This seems to be your first time using this "
+		                  "this version of BitlBee. Here's a list of new "
+		                  "features you may like to know about:\n\n%s\n",
+		                  irc->user->nick, msg );
+	
+	g_free( msg );
+	
+	g_snprintf( s, sizeof( s ), "%d", BITLBEE_VERSION_CODE );
+	set_setstr( &irc->b->set, "last_version", s );
+}
+
 /* IMPORTANT: Keep this list sorted! The short command logic needs that. */
-const command_t commands[] = {
+command_t root_commands[] = {
 	{ "account",        1, cmd_account,        0 },
 	{ "add",            2, cmd_add,            0 },
 	{ "allow",          1, cmd_allow,          0 },
@@ -1279,6 +1356,7 @@ const command_t commands[] = {
 	{ "help",           0, cmd_help,           0 }, 
 	{ "identify",       1, cmd_identify,       0 },
 	{ "info",           1, cmd_info,           0 },
+	{ "nick",           1, cmd_nick,           0 },
 	{ "no",             0, cmd_yesno,          0 },
 	{ "qlist",          0, cmd_qlist,          0 },
 	{ "register",       1, cmd_register,       0 },
@@ -1288,5 +1366,42 @@ const command_t commands[] = {
 	{ "set",            0, cmd_set,            0 },
 	{ "transfer",       0, cmd_transfer,       0 },
 	{ "yes",            0, cmd_yesno,          0 },
-	{ NULL }
+	/* Not expecting too many plugins adding root commands so just make a
+	   dumb array with some empty entried at the end. */
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ NULL },
+	{ NULL },
 };
+static const int num_root_commands = sizeof( root_commands ) / sizeof( command_t );
+
+gboolean root_command_add( const char *command, int params, void (*func)(irc_t *, char **args), int flags )
+{
+	int i;
+	
+	if( root_commands[num_root_commands-2].command )
+		/* Planning fail! List is full. */
+		return FALSE;
+	
+	for( i = 0; root_commands[i].command; i++ )
+	{
+		if( g_strcasecmp( root_commands[i].command, command ) == 0 )
+			return FALSE;
+		else if( g_strcasecmp( root_commands[i].command, command ) > 0 )
+			break;
+	}
+	memmove( root_commands + i + 1, root_commands + i,
+	         sizeof( command_t ) * ( num_root_commands - i - 1 ) );
+	
+	root_commands[i].command = g_strdup( command );
+	root_commands[i].required_parameters = params;
+	root_commands[i].execute = func;
+	root_commands[i].flags = flags;
+	
+	return TRUE;
+}
